#include "ble_rus.h"
#include "string.h"
#include <stdio.h>

/* Private definitions -------------------------------------------------------*/
#define RUS_BASE_UUID                   {{0xFB, 0x34, 0x9B, 0x4E, 0x90, 0x00, 0x00, 0x80, 0x00, 0x10, 0x00, 0x00, 0xFF, 0xFF, 0x00, 0x00}}
#define TIME_STAMP_BASE_UUID            RUS_BASE_UUID
#define RECORDING_UNIT_ID_BASE_UUID     RUS_BASE_UUID
#define DEVICE_ID_BASE_UUID             RUS_BASE_UUID

/* Private function prototypes -----------------------------------------------*/
uint32_t time_stamp_char_init(ble_rus_t* p_rus, 
                              const ble_rus_init_t* p_rus_init);

uint32_t recording_unit_id_char_init(ble_rus_t* p_rus, 
                                     const ble_rus_init_t* p_rus_init);

uint32_t device_id_char_init(ble_rus_t* p_rus, 
                             const ble_rus_init_t* p_rus_init);

/* Exported functions --------------------------------------------------------*/
uint32_t ble_rus_init(ble_rus_t* p_rus, const ble_rus_init_t* p_rus_init)
{
  uint32_t err_code;
  ble_uuid_t ble_uuid;
  //ble_uuid128_t rus_base_uuid = RUS_BASE_UUID;
  
  if ((p_rus == NULL) || (p_rus_init == NULL))
  {
    return NRF_ERROR_NULL;
  }
  
  /*err_code = sd_ble_uuid_vs_add(&rus_base_uuid, &p_rus->uuid_type);
  if(err_code != NRF_SUCCESS)
  {
    printf("Error@1\r\n");
    return err_code;
  }*/
  
  //ble_uuid.type = p_rus->uuid_type;
  //ble_uuid.uuid = BLE_UUID_RECORDING_UNIT_SERVICE;

  //printf("UUID type was: %d\r\n", ble_uuid.type);
  
  BLE_UUID_BLE_ASSIGN(ble_uuid, BLE_UUID_RECORDING_UNIT_SERVICE);
  
  //Add service to table
  err_code = sd_ble_gatts_service_add(BLE_GATTS_SRVC_TYPE_PRIMARY,
                                      &ble_uuid,
                                      &p_rus->service_handle);
  if (err_code != NRF_SUCCESS)
  {
    printf("Error@2\r\n");
    return err_code;
  }
  
  //Initialise and add characteristics to attribute table
  err_code = time_stamp_char_init(p_rus, p_rus_init);
  if (err_code != NRF_SUCCESS)
  {
    printf("Error@3\r\n");
    return err_code;
  }
  
  /*
  err_code = recording_unit_id_char_init(p_rus, p_rus_init);
  if (err_code != NRF_SUCCESS)
  {
    return err_code;
  }
  
  err_code = device_id_char_init(p_rus, p_rus_init);
  if (err_code != NRF_SUCCESS)
  {
    return err_code;
  }
  */
  
  printf("Leaving@4\r\n");
  return NRF_SUCCESS;
}

/* Private functions ---------------------------------------------------------*/

uint32_t time_stamp_char_init(ble_rus_t* p_rus, 
                              const ble_rus_init_t* p_rus_init)
{
  ble_gatts_char_md_t char_md;
  ble_gatts_char_pf_t present_fmt;
  ble_gatts_attr_t    attr_char_value;
  ble_uuid_t          ble_uuid;
  ble_gatts_attr_md_t attr_md;
  
  ble_uuid.type = p_rus->uuid_type;
  ble_uuid.uuid = BLE_UUID_RECORDING_UNIT_TIME_STAMP;
    
  memset(&char_md, 0, sizeof(char_md));
  uint8_t char_desc[] = "Injection Timestamp";
  
  memset(&present_fmt, 0, sizeof(present_fmt));
    
  present_fmt.format = BLE_GATT_CPF_FORMAT_UTF8S;
  present_fmt.exponent = NULL;
  present_fmt.unit = NULL;
  present_fmt.name_space = BLE_GATT_CPF_NAMESPACE_BTSIG;
  present_fmt.desc = BLE_GATT_CPF_NAMESPACE_DESCRIPTION_UNKNOWN;
    
  char_md.char_props.read             = 1;
  char_md.p_char_user_desc            = char_desc; //Only accepts non-null terminated strings
  char_md.char_user_desc_size         = strlen((char*)char_desc);
  char_md.char_user_desc_max_size     = 32;
  char_md.p_char_pf                   = &present_fmt;
  char_md.p_user_desc_md              = NULL;
  char_md.p_cccd_md                   = NULL;
  char_md.p_sccd_md                   = NULL;
    
  memset(&attr_md, 0, sizeof(attr_md));
    
  attr_md.read_perm  = p_rus_init->time_stamp_attr_md.read_perm;
  attr_md.write_perm = p_rus_init->time_stamp_attr_md.write_perm;
  attr_md.vloc       = BLE_GATTS_VLOC_STACK;
  attr_md.rd_auth    = 0;
  attr_md.wr_auth    = 0;
  attr_md.vlen       = 0;
    
  memset(&attr_char_value, 0, sizeof(attr_char_value));
    
  uint8_t* initial_time_stamp = "TOKI NO HAJIMARI";
  BLE_UUID_BLE_ASSIGN(ble_uuid, BLE_UUID_RECORDING_UNIT_TIME_STAMP);
    
  attr_char_value.p_uuid    = &ble_uuid;
  attr_char_value.p_attr_md = &attr_md;
  attr_char_value.init_len  = strlen((char*)initial_time_stamp);
  attr_char_value.init_offs = 0;
  attr_char_value.max_len   = 40;
  attr_char_value.p_value   = initial_time_stamp;
    
  return sd_ble_gatts_characteristic_add(p_rus->service_handle,
                                         &char_md,
                                         &attr_char_value,
                                         &p_rus->time_stamp_handles);
  }